#!/usr/bin/perl

# KMACPS Driver Installation Routine
# Version 1.4
# Januar 9th 2013
# Jan Heinecke
# Based on Version 1.3 from Markus Timpner [March 14th 2006]
#
# added DefaultDataDir and DefaultServerBin
# modified regex for extracting filter file name (remove white space)
# added chmod for filter file (0755)

use strict;

print "\nInstallation procedure for Konica Minolta drivers is starting.\n";

my @DefaultDataDir=();
$DefaultDataDir[0]="/usr/share/cups"; #default DataDir, if not defined in cupsd.conf
my @DefaultServerBin=();
$DefaultServerBin[0]="/usr/lib/cups"; # default ServerBin, if not defined in cupsd.conf
$DefaultServerBin[1]="/usr/libexec/";
$DefaultServerBin[2]="/usr/lib32/cups";

my $cupsconffile;
my $inifile="kmacpsdriverini.xml";
my $FilterOrg;
my $DataDir;
my $ServerBin;
my $PPDDir;
my $FilterDir;
my $DataDirSearch='DataDir\s+';
my $ServerBinSearch='ServerBin\s+';

my $i;
if (defined $ARGV[0])
{
	$cupsconffile=$ARGV[0];
	quotemeta($cupsconffile);
}
else{
	$cupsconffile="/etc/cups/cupsd.conf";
}
if ( (-e $cupsconffile) && (-r $cupsconffile)){
	open (CUPSCONF, $cupsconffile) or die "Can't open file: $cupsconffile\n";

	while (<CUPSCONF>)	{
		if (m/$DataDirSearch/i)		{
			chomp $_;
			my($tmp1 , $tmp2) = split /\s+/,$_,2;
			$DataDir = $tmp2;
			$PPDDir="$DataDir\/model";
			
		}
		if(m/$ServerBinSearch/i){
			chomp $_;
			my($tmp1 , $tmp2) = split /\s+/,$_,2;
			$ServerBin = $tmp2;
			$FilterDir="$ServerBin\/filter";
			
		}
	}

	close(CUPSCONF);
}
else{	
	if(-e $cupsconffile){
		file_noread($cupsconffile);
	}
	else{
		file_noexist($cupsconffile);
	}
}

if(!(defined($DataDir))){
  for ($i=0; $i<@DefaultDataDir;$i++)
  {
    if(-e $DefaultDataDir[$i]){
      $DataDir = $DefaultDataDir[$i];
      $PPDDir = "$DataDir\/model";
    }
  }
}

if(!(defined($ServerBin))){
  for ($i=0; $i<@DefaultServerBin;$i++)
  {
    if(-e $DefaultServerBin[$i]){
      $ServerBin = $DefaultServerBin[$i];
      $FilterDir = "$ServerBin\/filter";
    }
  }
}


if (!(-e $PPDDir)){
	file_noexist($PPDDir);
}
elsif( !(-w $PPDDir)){
	file_nowrite($PPDDir);
}
elsif(!(-e $FilterDir)){
print ("filterdir_noexist\n");
	file_noexist($FilterDir);
}
elsif(!(-w $FilterDir)){
	file_nowrite($FilterDir);
}
else{
	print "Installation starting\n";
	installPPD($PPDDir,getPPDFiles($inifile));
	installFilter($FilterDir,getFilterFiles($inifile));
	print "\nInstallation procedure finished!\n"; 
	
	print "\nTo install printers you can use the CUPS webinterface for example.\n";
	print "Please restart the CUPS server now\n\n";
	exit;
}

sub file_noread{
	my @filename = @_;
	print "You do not have sufficient priviledges to read $filename[0]\n";
	print "Maybe you are not root!\n";
	print "Exiting\n";
	exit;
}
sub file_nowrite{
	my @filename = @_;
	print "You do not have sufficient priviledges to write $filename[0]\n";
	print "Maybe you are not root!\n";
	print "Exiting\n";
	exit;
}
sub file_noexist{
	my @filename = @_;
	print "No such file: $filename[0]\n";
	print "usage:\t install [CUPS configuration file]\n";
	print "Exiting\n";
	exit;	
}
sub getPPDFiles{
	my $inifile=@_[0];
	my @ppdfiles=();
	open(INIFILE,$inifile);
	while(<INIFILE>){
		if(m/<ppdfile>/){
			s/<ppdfile>//;
			s/<\/ppdfile>//;
			s/\s//;
			s/\n//;
			push (@ppdfiles,$_);
		}
	}
	return @ppdfiles;		
}

sub getFilterFiles{
	my $inifile=@_[0];
	my @filterfiles=();
	open(INIFILE,$inifile);
	while(<INIFILE>){
		if(m/<filterfile>/){
			s/<filterfile>//;
			s/<\/filterfile>//;
			s/\s//;
			s/\n//;
			push (@filterfiles,$_);
		}
	}
	return @filterfiles;		
}

sub installPPD{

	my $PPDDir=shift @_;
	my @ppds=@_;
	my $ppd="";
	my $VersionCheck=0;
	my $newversion=0;
	my $oldversion=0; 
	print "Installing PPD files to $PPDDir\n";

	foreach $ppd (@ppds){
                  
        $ppd =~ s/^\"+//;
        $ppd =~ s/^\s+//;
        $ppd =~ s/\s+$//;
        
        print "\n\t$ppd\n";
        print "\t".getPPDModel("$ppd");
		$newversion=getPPDVersion("$ppd");
		if(-e "$PPDDir/$ppd"){
			$oldversion=getPPDVersion("$PPDDir/$ppd");
			if($newversion==0 || $oldversion==0){
				print "\tERROR:PPD cannot be installed\n";
				print "\tFile version cannot be obtained!\n";
			}
			if($newversion lt $oldversion){
				print "\told version: ".$oldversion."\n";
				print "\tnew version: ".$newversion."\n";
				print "The existing PPD is newer than the PPD selected for installation.\n";
				$VersionCheck=1;				
			}
			if($newversion eq $oldversion){
				print "\told version: ".$oldversion."\n";
				print "\tnew version: ".$newversion."\n";
				print "The existing PPD is the same version as the PPD selected for installation.\n";
						
				$VersionCheck=1;
			}
						
		}

		if($VersionCheck==0){
			print "\tInstalling ".$ppd." (".$newversion.")"; 	
			`cp $ppd $PPDDir`;	
		}
		else{
			print "Do you like to overwrite the existing PPD [y]? ";
			my $overwrite_decision="n";
			$overwrite_decision=<STDIN>;
			chomp $overwrite_decision;
			if(($overwrite_decision eq ('y' or 'Y'))or(length($overwrite_decision)==0)){
				print "\tOverwriting PPD version!\n";
				print "\tInstalling ".$ppd." (".$newversion.")\n"; 	
				`cp $ppd $PPDDir`;
			}
			else{
				print "PPD will not be installed.\n";
				print "Please be aware that the driver might not work correctly!\n";
			}

		}
			
	}

}

sub installFilter{
	my $FilterDir=shift @_;
	my @filters=@_;
	my $filter="";
	my $VersionCheck=0;
	my $newversion=0;
	my $oldversion=0; 

	print "\n\nInstalling Filter files to $FilterDir\n";
	foreach $filter (@filters){
        $filter =~ s/^\"+//;
        $filter =~ s/^\s+//;
        $filter =~ s/\s+$//;
		print "\n\t".$filter."\n";
		$newversion=getFilterVersion("$filter");
		if(-e "$FilterDir/$filter"){
			$oldversion=getFilterVersion("$FilterDir/$filter");
			if($newversion==0 || $oldversion==0){
				print "ERROR:Filter cannot be installed\n";
				print "File version cannot be obtained!\n";
			}
			if($newversion lt $oldversion){
				print "\told version: ".$oldversion."\n";
				print "\tnew version: ".$newversion."\n";
				print "The existing filter file is newer than the filter file selected for installation.\n";
				$VersionCheck=1;				
			}
			if($newversion eq $oldversion){
				print "\told version: ".$oldversion."\n";
				print "\tnew version: ".$newversion."\n";
				print "The existing filter file is the same version as the filter file selected for installation.\n";
				$VersionCheck=1;
			}
						
		}
		if($VersionCheck==0){
			copyFilter($filter,$FilterDir);
			print "\tInstalling ".$filter." (".$newversion.")"; 	
		}
		else{
			print "Do you like to overwrite the existing filter [y] ?";
			my $overwrite_decision="n";
			$overwrite_decision=<STDIN>;
			chomp $overwrite_decision;
			if(($overwrite_decision eq ('y' or 'Y'))or(length($overwrite_decision)==0)){
				print "Overwriting old filter version!\n";
				print "\tInstalling ".$filter." (".$newversion.")"; 	
				copyFilter($filter,$FilterDir);
			}
			else{
				print "Filter file will not be installed.\n";
				print "Please be aware that the driver might not work correctly!\n";
			}

		}
			
	}
}

sub getFilterVersion{ 
	my $filter=@_[0];
	my $version="";
	open(FILTER,$filter) or die "Cannot open $filter: $!";
	while(<FILTER>){
		if(m/Version/){
			s/\#//;			
			s/[ \t\r\n\v\f]+//; # remove any whitespace 
			s/\s+//;			
			s/\s+//;			

			my($versionmarker,$versionnumber)=split/:/,$_,2;
			$version=$versionnumber;
			last;
		}
					
	}
	close(FILTER);
	return $version;
				
}

sub copyFilter{
	my $filter=@_[0];
	my $FilterDir=@_[1];
        $filter =~ s/^\s+//;
        $filter =~ s/\s+$//;

	`cp $filter $FilterDir`;	
    chmod (0755,$FilterDir."/".$filter);
}

sub getPPDVersion{ 
	my $ppd=@_[0];
	my $version="";
	open(PPD,$ppd) or die "Cannot open $ppd: $!";
	while(<PPD>){
		if(m/FileVersion/){
			s/\*//;
			s/[ \t\r\n\v\f]+//; # remove any whitespace 
			s/\s+$//;			
			s/\"//;
			s/\"//;
			my($versionmarker,$versionnumber)=split/:/,$_,2;
			$version=$versionnumber;
			last;
		}
					
	}
	close(PPD);
	return $version;
				
}

sub getPPDModel{ 
	my $ppd=@_[0];
	my $version="";
	open(PPD,$ppd) or die "Cannot open $ppd: $!";
	while(<PPD>){
		if(m/ModelName/){
			s/\*//;
			s/[ \t\r\n\v\f]+//; # remove any whitespace 
			s/\"//;
			s/\"//;
			
			my($versionmarker,$versionnumber)=split/:/,$_,2;
			$version=$versionnumber;
			last;
		}
					
	}
	close(PPD);
	return $version;
				
}

